@@ -1,14 +1,16 @@
-import collections
 import logging
 from dataclasses import dataclass
 
 from pydantic import ValidationError
 
-from monarch_py.datamodels.model import Association, AssociationResults, Entity
+from monarch_py.datamodels.model import Association, AssociationResults, Entity, EntityResults
 from monarch_py.datamodels.solr import SolrQuery, core
 from monarch_py.interfaces.association_interface import AssociationInterface
 from monarch_py.interfaces.entity_interface import EntityInterface
+from monarch_py.interfaces.search_interface import SearchInterface
 from monarch_py.service.solr_service import SolrService
+from monarch_py.datamodels.solr import SolrQuery, core
+
 from monarch_py.utilities.utils import escape
 
 logger = logging.getLogger(__name__)
@@ -16,8 +18,8 @@ logger = logging.getLogger(__name__)
 
 @dataclass
 class SolrImplementation(
-    EntityInterface, AssociationInterface
-):  # todo: support SearchInterface
+    EntityInterface, AssociationInterface, SearchInterface
+):
     """Implementation of Monarch Interfaces for Solr endpoint"""
 
     base_url: str = "http://localhost:8983/solr"
@@ -26,9 +28,7 @@ class SolrImplementation(
     # Implements: EntityInterface #
     ###############################
 
-    def get_entity(
-        self, id: str, get_association_counts: bool = False, get_hierarchy: bool = False
-    ) -> Entity:
+    def get_entity(self, id: str) -> Entity:
         """Retrieve a specific entity by exact ID match, with optional extras
 
         Args:
@@ -43,7 +43,7 @@ class SolrImplementation(
         solr = SolrService(base_url=self.base_url, core=core.ENTITY)
         solr_document = solr.get(id)
         entity = Entity(**solr_document)
-
+        
         # todo: make an endpoint for getting facet counts?
         # if get_association_counts:
         #    entity["association_counts"] = self.get_entity_association_counts(id)
@@ -53,21 +53,6 @@ class SolrImplementation(
 
         return entity
 
-    def get_entity_association_counts(self, id: str):
-        """Returns a list and count of associations for an entity"""
-
-        solr = SolrService(base_url=self.base_url, core=core.ASSOCIATION)
-
-        object_categories = solr.get_filtered_facet(
-            id, filter_field="subject", facet_field="object_category"
-        )
-        subject_categories = solr.get_filtered_facet(
-            id, filter_field="object", facet_field="subject_category"
-        )
-        categories = collections.Counter(object_categories) + collections.Counter(
-            subject_categories
-        )
-        return categories
 
     ####################################
     # Implements: AssociationInterface #
@@ -136,9 +121,7 @@ class SolrImplementation(
                 logger.error(f"Validation error for {doc}")
                 raise
 
-        results = AssociationResults(
-            limit=limit, offset=offset, total=total, associations=associations
-        )
+        results = AssociationResults(associations=associations, limit=limit, offset=offset, total=total)
 
         return results
 
@@ -146,34 +129,44 @@ class SolrImplementation(
     # Implements: SearchInterface
     # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
+    def search(
+            self,
+            q: str = "*:*",
+            category: str = None,
+            taxon: str = None,
+            offset: int = 0,
+            limit: int = 20
+    ) -> EntityResults:
+        """Search for entities by label, with optional filters """
+
+        solr = SolrService(base_url=self.base_url, core=core.ENTITY)
+        query = SolrQuery(start=offset, rows=limit)
+
+        query.q = q
+
+        query.query_fields = "id^100 name^10 synonym"
+        query.def_type = "edismax"
+
+        if category:
+            query.add_field_filter_query("category", category)
+        if taxon:
+            query.add_field_filter_query("in_taxon", taxon)
+
+        query_result = solr.query(query)
+        total = query_result.response.num_found
+
+        entities = []
+        for doc in query_result.response.docs:
+            try:
+                entity = Entity(**doc)
+                entities.append(entity)
+            except ValidationError:
+                logger.error(f"Validation error for {doc}")
+                raise
+
+        results = EntityResults(
+            limit=limit, offset=offset, total=total, entities=entities
+        )
+
+        return results
 
-# def get_node_hierarchy(entity_id):
-#     superClasses = f""
-#
-#     # query_params = {
-#     #     q: str = "*:*",
-#     #     offset: int = 0,
-#     #     limit: int = 20,
-#     #     category: str = None,
-#     #     predicate: str = None,
-#     #     subject: str = None,
-#     #     object: str = None,
-#     #     entity: str = None, # return nodes where entity is subject or object
-#     #     between: str = None
-#     # }
-#
-#     query = build_association_query(
-#         {
-#             #'q':'*:*',
-#             "entity": f'"{entity_id}"',
-#             "predicate": "biolink:same_as",
-#         }
-#     )
-#     equivalentClasses = requests.get(f"{solr_url}/association/select{query}").json()
-#
-#     subClasses = ""
-#     return {
-#         "superClasses": superClasses,
-#         "equivalentClasses": equivalentClasses,
-#         "subClasses": subClasses,
-#     }
