@@ -1,13 +1,8 @@
-from typing import Literal
-
 import docker
-import pystow
-
 
 SOLR_DATA_URL = "https://data.monarchinitiative.org/monarch-kg-dev/latest/solr.tar.gz"
 SQL_DATA_URL = "https://data.monarchinitiative.org/monarch-kg-dev/latest/monarch-kg.db.gz"
 
-
 def strip_json(doc: dict, *fields_to_remove: str):
     for field in fields_to_remove:
         try:
@@ -21,12 +16,8 @@ def escape(value: str) -> str:
     return value.replace(":", "\:")
 
 
-def check_for_data(data: Literal['solr', 'sql']):
-    data_dir = pystow.join("monarch", data)
-    return (data_dir.is_dir() and any(data_dir.iterdir()))
-
-
 def check_for_solr():
+    print("\nChecking for solr container...")
     dc = docker.from_env()
     c = dc.containers.list(all=True, filters={"name":"monarch_solr"})
     return None if not c else c[0]
